@@ -1,4 +1,5 @@
 # encoding: ASCII-8BIT
+# frozen_string_literal: false
 
 require_relative 'common'
 require 'net/ssh/buffer'
@@ -55,7 +56,7 @@ class TestBuffer < NetSSHTest
   end
 
   def test_from_with_array_argument_should_write_multiple_of_the_given_type
-    buffer = Net::SSH::Buffer.from(:byte, [1,2,3,4,5])
+    buffer = Net::SSH::Buffer.from(:byte, [1, 2, 3, 4, 5])
     assert_equal "\1\2\3\4\5", buffer.to_s
   end
 
@@ -318,16 +319,30 @@ class TestBuffer < NetSSHTest
   def test_write_dss_key_should_write_argument_to_end_of_buffer
     buffer = new("start")
 
-    key = OpenSSL::PKey::DSA.new
-    if key.respond_to?(:set_pqg)
-      key.set_pqg(0xffeeddccbbaa9988, 0x7766554433221100, 0xffddbb9977553311)
-      key.set_key(0xeeccaa8866442200, nil)
-    else
-      key.p = 0xffeeddccbbaa9988
-      key.q = 0x7766554433221100
-      key.g = 0xffddbb9977553311
-      key.pub_key = 0xeeccaa8866442200
-    end
+    p = 0xffeeddccbbaa9988
+    q = 0x7766554433221100
+    g = 0xffddbb9977553311
+    pub_key = 0xeeccaa8866442200
+
+    asn1 = OpenSSL::ASN1::Sequence.new(
+      [
+        OpenSSL::ASN1::Sequence.new(
+          [
+            OpenSSL::ASN1::ObjectId.new('DSA'),
+            OpenSSL::ASN1::Sequence.new(
+              [
+                OpenSSL::ASN1::Integer.new(p),
+                OpenSSL::ASN1::Integer.new(q),
+                OpenSSL::ASN1::Integer.new(g)
+              ]
+            )
+          ]
+        ),
+        OpenSSL::ASN1::BitString.new(OpenSSL::ASN1::Integer.new(pub_key).to_der)
+      ]
+    )
+
+    key = OpenSSL::PKey::DSA.new(asn1.to_der)
 
     buffer.write_key(key)
     assert_equal "start\0\0\0\7ssh-dss\0\0\0\011\0\xff\xee\xdd\xcc\xbb\xaa\x99\x88\0\0\0\010\x77\x66\x55\x44\x33\x22\x11\x00\0\0\0\011\0\xff\xdd\xbb\x99\x77\x55\x33\x11\0\0\0\011\0\xee\xcc\xaa\x88\x66\x44\x22\x00", buffer.to_s
@@ -336,13 +351,17 @@ class TestBuffer < NetSSHTest
   def test_write_rsa_key_should_write_argument_to_end_of_buffer
     buffer = new("start")
 
-    key = OpenSSL::PKey::RSA.new
-    if key.respond_to?(:set_key)
-      key.set_key(0x7766554433221100, 0xffeeddccbbaa9988, nil)
-    else
-      key.e = 0xffeeddccbbaa9988
-      key.n = 0x7766554433221100
-    end
+    n = 0x7766554433221100
+    e = 0xffeeddccbbaa9988
+
+    asn1 = OpenSSL::ASN1::Sequence(
+      [
+        OpenSSL::ASN1::Integer(n),
+        OpenSSL::ASN1::Integer(e)
+      ]
+    )
+
+    key = OpenSSL::PKey::RSA.new(asn1.to_der)
 
     buffer.write_key(key)
     assert_equal "start\0\0\0\7ssh-rsa\0\0\0\011\0\xff\xee\xdd\xcc\xbb\xaa\x99\x88\0\0\0\010\x77\x66\x55\x44\x33\x22\x11\x00", buffer.to_s
@@ -446,7 +465,7 @@ class TestBuffer < NetSSHTest
   end
 
   def random_ecdsa_sha2_nistp256
-    k = OpenSSL::PKey::EC.new('prime256v1').generate_key
+    k = OpenSSL::PKey::EC.generate('prime256v1')
     buffer = Net::SSH::Buffer.from(:string, 'nistp256',
                                    :string, k.public_key.to_bn.to_s(2))
     key = yield(buffer)
@@ -455,7 +474,7 @@ class TestBuffer < NetSSHTest
   end
 
   def random_ecdsa_sha2_nistp384
-    k = OpenSSL::PKey::EC.new('secp384r1').generate_key
+    k = OpenSSL::PKey::EC.generate('secp384r1')
     buffer = Net::SSH::Buffer.from(:string, 'nistp384',
                                    :string, k.public_key.to_bn.to_s(2))
     key = yield(buffer)
@@ -464,7 +483,7 @@ class TestBuffer < NetSSHTest
   end
 
   def random_ecdsa_sha2_nistp521
-    k = OpenSSL::PKey::EC.new('secp521r1').generate_key
+    k = OpenSSL::PKey::EC.generate('secp521r1')
     buffer = Net::SSH::Buffer.from(:string, 'nistp521',
                                    :string, k.public_key.to_bn.to_s(2))
     key = yield(buffer)
