@@ -1,11 +1,10 @@
 # encoding: ASCII-8BIT
 
-require 'common'
+require_relative '../common'
 require 'timeout'
 require 'net/ssh/transport/packet_stream'
 
 module Transport
-
   class TestPacketStream < NetSSHTest # rubocop:disable Metrics/ClassLength
     include Net::SSH::Transport::Constants
 
@@ -58,12 +57,12 @@ module Transport
     end
 
     def test_available_for_read_should_return_nontrue_when_self_is_not_ready
-      IO.expects(:select).with([stream], nil, nil, 0).returns([[],[],[]])
+      IO.expects(:select).with([stream], nil, nil, 0).returns([[], [], []])
       assert !stream.available_for_read?
     end
 
     def test_available_for_read_should_return_true_when_self_is_ready
-      IO.expects(:select).with([stream], nil, nil, 0).returns([[self],[],[]])
+      IO.expects(:select).with([stream], nil, nil, 0).returns([[self], [], []])
       assert stream.available_for_read?
     end
 
@@ -135,7 +134,7 @@ module Transport
 
     def test_next_packet_should_eventually_return_packet_when_non_blocking_and_partial_read
       IO.stubs(:select).returns([[stream]])
-      stream.stubs(:recv).returns(packet[0,10], packet[10..-1])
+      stream.stubs(:recv).returns(packet[0, 10], packet[10..-1])
       assert_nil stream.next_packet(:nonblock)
       packet = stream.next_packet(:nonblock)
       assert_not_nil packet
@@ -160,7 +159,7 @@ module Transport
 
     def test_next_packet_should_block_when_requested_until_entire_packet_is_available
       IO.stubs(:select).returns([[stream]])
-      stream.stubs(:recv).returns(packet[0,10], packet[10,20], packet[20..-1])
+      stream.stubs(:recv).returns(packet[0, 10], packet[10, 20], packet[20..-1])
       packet = stream.next_packet(:block)
       assert_not_nil packet
       assert_equal DEBUG, packet.type
@@ -182,7 +181,7 @@ module Transport
     end
 
     def test_send_packet_should_enqueue_and_send_data_immediately
-      stream.expects(:send).times(3).with { |a,b| a == stream.write_buffer && b == 0 }.returns(15)
+      stream.expects(:send).times(3).with { |a, b| a == stream.write_buffer && b == 0 }.returns(15)
       IO.expects(:select).times(2).returns([[], [stream]])
       stream.send_packet(ssh_packet)
       assert !stream.pending_write?
@@ -204,6 +203,12 @@ module Transport
     end
 
     PACKETS = {
+      'chacha20-poly1305@openssh.com' => {
+        'implicit' => {
+          false => ["5aa01d1e3eff7c277d19f111a384b229fec8652db616d9350d6ef9f51f2011637b60d406673fffad5a647eba"].pack('H*'),
+          :standard => ["5aa01d263f83e1451c7d981526aa8e03b3ec44857a5dde471d76ba92fd92c9a77911c43ca96a13e37a5b1a346508016793f4a57a"].pack('H*')
+        }
+      },
       "3des-cbc" => {
         "hmac-md5" => {
           false => "\003\352\031\261k\243\200\204\301\203]!\a\306\217\201\a[^\304\317\322\264\265~\361\017\n\205\272, \000\032w\312\t\306\374\271\345p\215\224\373\363\v\261",
@@ -1052,8 +1057,9 @@ module Transport
       PACKETS[key].merge!(val)
     end
 
-    ciphers = Net::SSH::Transport::CipherFactory::SSH_TO_OSSL.keys
-    hmacs = Net::SSH::Transport::HMAC::MAP.keys
+    ciphers = Net::SSH::Transport::CipherFactory::SSH_TO_OSSL.keys + Net::SSH::Transport::CipherFactory::SSH_TO_CLASS.keys
+    hmacs = Net::SSH::Transport::HMAC::MAP.keys + ["implicit"]
+    implicit_ciphers = ["chacha20-poly1305@openssh.com"]
 
     ciphers.each do |cipher_name|
       unless Net::SSH::Transport::CipherFactory.supported?(cipher_name) && PACKETS.key?(cipher_name)
@@ -1070,13 +1076,22 @@ module Transport
 
       hmacs.each do |hmac_name|
         [false, :standard].each do |compress|
+          next if (hmac_name != "implicit" && implicit_ciphers.include?(cipher_name)) ||
+                  (hmac_name == "implicit" && !implicit_ciphers.include?(cipher_name))
+
           cipher_method_name = cipher_name.gsub(/\W/, "_")
           hmac_method_name   = hmac_name.gsub(/\W/, "_")
 
           define_method("test_next_packet_with_#{cipher_method_name}_and_#{hmac_method_name}_and_#{compress}_compression") do
             opts = { shared: "123", hash: "^&*", digester: OpenSSL::Digest::SHA1 }
-            cipher = Net::SSH::Transport::CipherFactory.get(cipher_name, opts.merge(key: "ABC", decrypt: true, iv: "abc"))
-            hmac = Net::SSH::Transport::HMAC.get(hmac_name, "{}|", opts)
+            key = "ABC"
+            cipher = Net::SSH::Transport::CipherFactory.get(cipher_name, opts.merge(key: key, decrypt: true, iv: "abc"))
+            hmac =
+              if cipher.implicit_mac?
+                cipher.implicit_mac
+              else
+                Net::SSH::Transport::HMAC.get(hmac_name, "{}|", opts)
+              end
 
             stream.server.set cipher: cipher, hmac: hmac, compression: compress
             stream.stubs(:recv).returns(PACKETS[cipher_name][hmac_name][compress])
@@ -1092,8 +1107,14 @@ module Transport
 
           define_method("test_enqueue_packet_with_#{cipher_method_name}_and_#{hmac_method_name}_and_#{compress}_compression") do
             opts = { shared: "123", digester: OpenSSL::Digest::SHA1, hash: "^&*" }
-            cipher = Net::SSH::Transport::CipherFactory.get(cipher_name, opts.merge(key: "ABC", iv: "abc", encrypt: true))
-            hmac = Net::SSH::Transport::HMAC.get(hmac_name, "{}|", opts)
+            key = "ABC"
+            cipher = Net::SSH::Transport::CipherFactory.get(cipher_name, opts.merge(key: key, iv: "abc", encrypt: true))
+            hmac =
+              if cipher.implicit_mac?
+                cipher.implicit_mac
+              else
+                Net::SSH::Transport::HMAC.get(hmac_name, "{}|", opts)
+              end
 
             srand(100)
             stream.client.set cipher: cipher, hmac: hmac, compression: compress
@@ -1129,5 +1150,4 @@ module Transport
       end
     end
   end
-
 end
